import os
import signal
import subprocess
import urllib.request

from tqdm import tqdm


def download_with_progress_bar(url, filename):
    # Extract the basename of the file
    base_filename = os.path.basename(filename)

    # Define the reporthook function
    def reporthook(block_num, block_size, total_size):
        if pbar.total is None:
            pbar.total = total_size

        pbar.update(block_size)

    # Create a tqdm progress bar instance with a custom bar_format
    pbar = tqdm(
        unit="B",
        unit_scale=True,
        miniters=1,
        desc=base_filename,
        bar_format="{l_bar}{bar}| {n_fmt}/{total_fmt} [{percentage:.0f}%]",
    )

    # Use urlretrieve to download the file with the reporthook
    urllib.request.urlretrieve(url, filename, reporthook)

    # Close the progress bar
    pbar.close()


def start_shell_script(file_path, *args):
    try:
        # Construct the command with the script and the additional arguments
        command = ["sh", file_path] + list(args)

        # Open the script and redirect its stdout and stderr to log files for debugging
        with open("script_output.log", "w") as out, open(
            "script_error.log", "w"
        ) as err:
            process = subprocess.Popen(
                command,
                stdout=out,
                stderr=err,
                preexec_fn=os.setpgrp,  # Start the process in a new process group
            )

        print(f"Started the file: {file_path} with PID: {process.pid}")

        return process

    except FileNotFoundError:
        print(f"File not found: {file_path}")

    except Exception as e:
        print(f"An error occurred: {e}")


def stop_shell_script(pid):
    try:
        # Kill the process group to ensure all child processes are terminated
        os.killpg(os.getpgid(pid), signal.SIGTERM)
        print(f"Stopped the process with PID: {pid}")

    except Exception as e:
        print(f"An error occurred: {e}")



import json
import os
from contextlib import asynccontextmanager
from enum import Enum
from pathlib import Path

import uvicorn
from connexion import AsyncApp, ConnexionMiddleware
from connexion.options import SwaggerUIOptions
from fastapi import FastAPI, Request
from prefect import flow

from timestep.api.openai.v1.controllers.completions_controller import create_completion
from timestep.utils import download_with_progress_bar, start_shell_script

default_llamafile_filename = "TinyLlama-1.1B-Chat-v1.0.F16.llamafile"
default_llamafile_url = f"https://huggingface.co/Mozilla/TinyLlama-1.1B-Chat-v1.0-llamafile/resolve/main/{default_llamafile_filename}?download=true"

host = "0.0.0.0"  # TODO: move to config
llamafile_path = f"./models/{default_llamafile_filename}"  # TODO: namespace under llamafile, include port, etc.
port = 8080

connexion_app = AsyncApp(import_name=__name__)


@asynccontextmanager
async def lifespan(app: FastAPI):
    agent_flow = await flow.from_source(
        source=str(Path(__file__).parent),
        entrypoint="worker.py:agent_flow",
    )

    try:
        await agent_flow.deploy(
            name="agent-flow-deployment",
            # parameters=dict(name="Marvin"),
            # work_pool_name="local",
            work_pool_name="default",
        )

    except Exception as e:
        print(
            f"{e} - Recommendation: `prefect server start` and `prefect worker start --pool default`"
        )

    if os.path.basename(
        llamafile_path
    ) == default_llamafile_filename and not os.path.exists(llamafile_path):
        os.makedirs(os.path.dirname(llamafile_path), exist_ok=True)
        download_with_progress_bar(default_llamafile_url, llamafile_path)

    assert os.path.exists(llamafile_path)

    process = start_shell_script(
        llamafile_path,
        "--host",
        host,
        "--nobrowser",
        "--path",
        "/zip/llama.cpp/server/public",
        "--port",
        f"{port}",
        "--temp",
        "0.0",
    )

    yield

    print(f"Terminating llamafile with PID: {process.pid}")
    process.terminate()


fastapi_app = FastAPI(
    lifespan=lifespan,
    # servers= # TODO: add server for testing?
)

connexion_app.add_api(
    "api/ap/v1/openapi/openapi.yaml",
    pythonic_params=True,
    resolver_error=501,
    swagger_ui_options=SwaggerUIOptions(serve_spec=False),
)

connexion_app.add_api(
    "api/openai/v1/openapi/openapi.yaml",
    base_path="/openai/v1",
    pythonic_params=True,
    resolver_error=500,
)

fastapi_app.mount("/api", ConnexionMiddleware(app=connexion_app, import_name=__name__))


class Engine(str, Enum):
    copilot_codex = "copilot-codex"


@fastapi_app.get("/v1/engines")
async def list_engines():
    # return [engine.value for engine in Engine]
    return {
        "data": [
            {
                "id": Engine.copilot_codex,
                "name": "Copilot Codex",
                "description": "OpenAI's Codex model, formerly known as GitHub Copilot",
            }
        ]
    }


@fastapi_app.post("/v1/engines/{engine}/completions")
async def create_code_completion(engine: Engine, request: Request):
    print(
        f"=== {__name__}.create_code_completion(engine: Engine, request: Request) ==="
    )
    print(f"engine: {engine}")
    print(f"request: {request}")

    if engine == Engine.copilot_codex:
        try:
            body = await request.json()

        except json.decoder.JSONDecodeError as e:
            print(f"Error: {e}")
            body = {
                "model": "LLaMA_CPP",
                "prompt": "int main() {",
            }

        # body["model"] = "LLaMA_CPP"

        return await create_completion(body, token_info={}, user=None)

    else:
        raise NotImplementedError("Not implemented")

    # @fastapi_app.get("/v2/models/{model_id}")
    # async def create_code_completion(model_id: str, request: Request):
    #     # model_info = models_service.retrieve_model(model_id, token_info, user)
    #     model_info = model_service.retrieve_model(model_id)

    #     print("model_info: ", model_info)

    #     return model_info


def main(*args, **kwargs):
    print(f"=== {__name__}.main(*args, **kwargs) ===")
    print(f"args: {args}")
    print(f"kwargs: {kwargs}")

    cwd = os.getcwd()
    print(f"cwd: {cwd}")

    uvicorn.run(
        f"{__name__}:fastapi_app",
        host="0.0.0.0",
        log_level="info",
        loop="asyncio",
        port=8000,
        reload=True,
        reload_dirs=[
            f"{os.getcwd()}/timestep",
        ],
        workers=1,
    )


if __name__ == "__main__":
    main()



import asyncio
import json
import time
import uuid
from typing import List, Optional

from openai.pagination import AsyncCursorPage
from openai.types.beta.assistant import Assistant
from openai.types.beta.assistant_deleted import AssistantDeleted
from openai.types.beta.assistant_update_params import AssistantUpdateParams
from openai.types.beta.thread import Thread
from openai.types.beta.threads.message import Message, MessageContent
from openai.types.beta.threads.run import Run
from openai.types.beta.threads.text import Text
from openai.types.beta.threads.text_content_block import TextContentBlock
from prefect.deployments import run_deployment
from prefect.deployments.flow_runs import FlowRun
from sse_starlette import EventSourceResponse

from timestep.services import agent_service

# from timestep.worker import step

# TODO: I see this OpenAI-Beta header in the openai spec, e.g.:
# curl https://api.openai.com/v1/assistants/asst_abc123 \
# -H "Content-Type: application/json" \
# -H "Authorization: Bearer $OPENAI_API_KEY" \
# -H "OpenAI-Beta: assistants=v2" \
# -d '{
#     "instructions": "You are an HR bot, and you have access to files to answer employee questions about company policies. Always response with info from either of the files.",
#     "tools": [{"type": "file_search"}],
#     "model": "gpt-4-turbo"
#     }'
# do I need to handle this in the API?


async def cancel_run(*args, **kwargs):
    """Cancels a run that is &#x60;in_progress&#x60;.

    :param thread_id: The ID of the thread to which this run belongs.
    :type thread_id: str
    :param run_id: The ID of the run to cancel.
    :type run_id: str

    :rtype: Union[RunObject, Tuple[RunObject, int], Tuple[RunObject, int, Dict[str, str]]
    """
    print("=== cancel_run ===")
    print("args: ", args)
    print("kwargs: ", kwargs)

    raise NotImplementedError


# def create_assistant(create_assistant_request):  # noqa: E501
async def create_assistant(body, token_info: dict, user: str):
    """Create an assistant with a model and instructions.

     # noqa: E501

    :param create_assistant_request:
    :type create_assistant_request: dict | bytes

    :rtype: Union[AssistantObject, Tuple[AssistantObject, int], Tuple[AssistantObject, int, Dict[str, str]]
    """
    # if connexion.request.is_json:
    #     create_assistant_request = CreateAssistantRequest.from_dict(connexion.request.get_json())  # noqa: E501

    print("=== create_assistant ===")

    assistant = Assistant(
        id=str(uuid.uuid4()),
        created_at=int(time.time()),
        description=body.get("description"),
        instructions=body.get("instructions"),
        model=body.get("model"),
        name=body.get("name"),
        object="assistant",
        tools=body.get("tools", []),
    )

    print("assistant: ", assistant)

    await agent_service.insert_agent(agent=assistant)

    return assistant.model_dump(mode="json")


# def create_message(thread_id, create_message_request):  # noqa: E501
async def create_message(body, token_info, thread_id, user):
    """Create a message.

     # noqa: E501

    :param thread_id: The ID of the [thread](/docs/api-reference/threads) to create a message for.
    :type thread_id: str
    :param create_message_request:
    :type create_message_request: dict | bytes

    :rtype: Union[MessageObject, Tuple[MessageObject, int], Tuple[MessageObject, int, Dict[str, str]]
    """
    # if connexion.request.is_json:
    #     create_message_request = CreateMessageRequest.from_dict(connexion.request.get_json())  # noqa: E501

    print("=== create_message ===")

    # thread = instance_store._shared_instance_state["threads"][thread_id]
    thread: Thread = Thread(
        **await get_thread(token_info=token_info, thread_id=thread_id, user=user)
    )

    content: List[MessageContent] = []

    if body.get("content"):
        content.append(
            TextContentBlock(
                text=Text(
                    annotations=[],
                    value=(
                        body.get("content")
                        if type(body.get("content")) == str
                        else body.get("content")[0].get("text")
                    ),
                ),
                type="text",
            )
        )

    message = Message(
        id=str(uuid.uuid4()),
        attachments=[],
        content=content,
        created_at=int(time.time()),
        object="thread.message",
        role=body.get("role"),
        thread_id=thread.id,
        status="incomplete",
    )

    print("message: ", message)

    instance_store = InstanceStoreSingleton()
    instance_store._shared_instance_state["messages"][message.id] = message

    return message.model_dump(mode="json")


# TODO: move this somewhere more appropriate
# maybe get rid of BackgroundTask and just trigger a run deployment
# @flow(log_prints=True)
# @cf.flow
# async def run_run(run_id: str, token_info: dict, thread_id: str, user: str):
#     print('=== run_run ===')
#     run: Run = Run(**await get_run(run_id=run_id, token_info=token_info, thread_id=thread_id, user=user))
#     assistant: Assistant = Assistant(**await get_assistant(assistant_id=run.assistant_id, token_info=token_info, user=user))
#     thread: Thread = Thread(**await get_thread(token_info=token_info, thread_id=thread_id, user=user))

#     # # TODO: use ControlFlow to run_once...
#     # @cf.flow
#     # def run_run_flow():
#     #     agent = cf.Agent(
#     #         name=assistant.name,
#     #         description=assistant.description,
#     #         instructions=assistant.instructions,
#     #         # memory
#     #         # tools=,
#     #         # ...
#     #     )

#     #     task = cf.Task(
#     #         "Tell me something I don't know",
#     #         # objective=
#     #         # instructions=
#     #         agents=[agent]
#     #     )

#     #     return task

#     # result = run_run_flow()

#     # print('result: ', result)

#     # messages: List[Message] = list_messages(
#     # SyncCursorPage[Message] = await list_messages(
#     list_messages_response = await list_messages(
#         limit=-1,
#         order='asc',
#         token_info=token_info,
#         thread_id=thread_id,
#         user=user,
#     )

#     messages = list_messages_response.get("data")

#     system_message = {
#         "content": "You are a helpful assistant with access to tools. Please be honest and do not hallucinate.",
#         "role": "system",
#     }

#     messages = [system_message] + messages
#     print('messages: ', messages)

#     def get_message_content(message):
#         content = message.get("content")

#         if type(content) == list:
#             if message.get("role") == "user":
#                 return [{
#                     "text": _content.get("text").get("value"),
#                     "type": _content.get("type"),
#                 } for _content in content]

#             else:
#                 assert len(content) == 1
#                 return content[0].get("text").get("value")

#         else:
#             return content

#     body = { # CompletionCreateParamsNonStreaming
#         "messages": [ {
#             "content": get_message_content(message),
#             "role": message.get("role"),
#         } for message in messages ],
#         "model": assistant.model,
#         "tools": assistant.tools,
#     }
#     print('body: ', body)

#     chat_completion: ChatCompletion = ChatCompletion(**await create_chat_completion(body=body, token_info=token_info, user=user)) # TODO: handle streaming use case
#     print('chat_completion: ', chat_completion)

#     choice = chat_completion.choices[0]
#     print('choice: ', choice)

#     finish_reason = choice.finish_reason
#     print('finish_reason: ', finish_reason) # "stop", "length", "tool_calls", "content_filter", "function_call"

#     response_message = chat_completion.choices[0].message

#     tool_calls = response_message.tool_calls

#     if tool_calls:
#         raise NotImplementedError

#     await create_message(body={
#         "content": response_message.content,
#         "role": response_message.role,
#     }, token_info=token_info, thread_id=thread_id, user=user)

#     # time.sleep(1)  # wait for the process to finish

#     modify_run_request = {
#         "status": "completed",
#     }

#     await modify_run(modify_run_request=modify_run_request, run_id=run_id, token_info=token_info, thread_id=thread_id, user=user)

#     # run: Run = Run(**await get_run(run_id=run_id, token_info=token_info, thread_id=thread_id, user=user))

#     # return run


# def create_run(thread_id, create_run_request):  # noqa: E501
# def create_run(body, token_info, user, thread_id):
async def create_run(body, token_info, thread_id, user):
    """Create a run.

     # noqa: E501

    :param thread_id: The ID of the thread to run.
    :type thread_id: str
    :param create_run_request:
    :type create_run_request: dict | bytes

    :rtype: Union[RunObject, Tuple[RunObject, int], Tuple[RunObject, int, Dict[str, str]]
    """
    # if connexion.request.is_json:
    #     create_run_request = CreateRunRequest.from_dict(connexion.request.get_json())  # noqa: E501

    print("=== create_run ===")
    print("body: ", body)

    stream = body.get("stream", False)
    assistant_id = body.get("assistant_id")

    assistant: Assistant = Assistant(
        **await get_assistant(
            assistant_id=assistant_id, token_info=token_info, user=user
        )
    )
    thread: Thread = Thread(
        **await get_thread(token_info=token_info, thread_id=thread_id, user=user)
    )

    run = Run(
        id=str(uuid.uuid4()),
        assistant_id=assistant.id,
        created_at=int(time.time()),
        instructions=assistant.instructions,
        model=assistant.model,
        object="thread.run",
        parallel_tool_calls=False,
        status="queued",
        thread_id=thread.id,
        tools=assistant.tools,
    )

    print("run: ", run)
    run_id = run.id

    instance_store = InstanceStoreSingleton()
    instance_store._shared_instance_state["runs"][run.id] = run

    flow_run: FlowRun = await run_deployment(
        idempotency_key=run.id,
        name="agent-flow/agent-flow-deployment",
        parameters={
            "inputs": {
                "run_id": run_id,
            }
        },
        # job_variables={"env": {"MY_ENV_VAR": "staging"}},
        timeout=0,  # don't wait for the run to finish
    )

    print("flow_run: ", flow_run)

    if stream:

        async def run_event_publisher():
            run: Run = Run(
                **await get_run(
                    run_id=run_id, token_info=token_info, thread_id=thread_id, user=user
                )
            )

            # response: Iterator[CreateChatCompletionStreamResponse] = model.create_chat_completion(**create_chat_completion_kwargs)
            # response: Generator[ChatCompletionChunk] = model.create_chat_completion_openai_v1(**create_chat_completion_kwargs)
            # response: Stream[ChatCompletionChunk] = model.astream(input=messages)
            await step(
                run_id=run.id, token_info=token_info, thread_id=run.thread_id, user=user
            )

            run: Run = Run(
                **await get_run(
                    run_id=run.id, token_info=token_info, thread_id=thread_id, user=user
                )
            )

            try:
                yield json.dumps(run.model_dump(mode="json"))

                # for chunk in response:
                # # async for chunk in response:
                #     chunk = ChatCompletionChunk(**chunk)
                #     # yield json.dumps(chunk)
                #     yield json.dumps(chunk.model_dump(mode="json"))
                #     # yield ChatCompletionChunk(**chunk).model_dump(mode="json")
                #     # yield chunk.model_dump(mode="json")

            except asyncio.CancelledError as e:
                # print(f"Disconnected from client (via refresh/close) {req.client}")
                print(f"Disconnected from client (via refresh/close)")
                # Do any other cleanup, if any
                raise e

        # print(f'=== RETURN: {__name__}.create_chat_completion(body: CompletionCreateParams, token_info: dict, user: str)) ===')
        return EventSourceResponse(run_event_publisher())

    else:
        # task = BackgroundTask(step, run_id=run.id, token_info=token_info, thread_id=run.thread_id, user=user)

        # return JSONResponse(run.model_dump(mode="json"), background=task)
        return run.model_dump(mode="json")


# def create_thread(create_thread_request=None):  # noqa: E501
async def create_thread(*args, **kwargs):
    """Create a thread.

     # noqa: E501

    :param create_thread_request:
    :type create_thread_request: dict | bytes

    :rtype: Union[ThreadObject, Tuple[ThreadObject, int], Tuple[ThreadObject, int, Dict[str, str]]
    """
    # if connexion.request.is_json:
    #     create_thread_request = CreateThreadRequest.from_dict(connexion.request.get_json())  # noqa: E501

    print("=== create_thread ===")
    print("args: ", args)
    print("kwargs: ", kwargs)

    thread = Thread(
        id=str(uuid.uuid4()),
        created_at=int(time.time()),
        object="thread",
        # tool_resources
    )

    print("thread: ", thread)

    instance_store = InstanceStoreSingleton()
    instance_store._shared_instance_state["threads"][thread.id] = thread

    return thread.model_dump(mode="json")


# def create_thread_and_run(create_thread_and_run_request):  # noqa: E501
async def create_thread_and_run(*args, **kwargs):
    """Create a thread and run it in one request.

     # noqa: E501

    :param create_thread_and_run_request:
    :type create_thread_and_run_request: dict | bytes

    :rtype: Union[RunObject, Tuple[RunObject, int], Tuple[RunObject, int, Dict[str, str]]
    """
    # if connexion.request.is_json:
    #     create_thread_and_run_request = CreateThreadAndRunRequest.from_dict(connexion.request.get_json())  # noqa: E501

    print("=== create_thread_and_run ===")
    print("args: ", args)
    print("kwargs: ", kwargs)

    raise NotImplementedError


# def delete_assistant(assistant_id):  # noqa: E501
async def delete_assistant(assistant_id: str, token_info: dict, user: str):
    """Delete an assistant.

     # noqa: E501

    :param assistant_id: The ID of the assistant to delete.
    :type assistant_id: str

    :rtype: Union[DeleteAssistantResponse, Tuple[DeleteAssistantResponse, int], Tuple[DeleteAssistantResponse, int, Dict[str, str]]
    """
    print("=== delete_assistant ===")
    # print('args: ', args)
    # print('kwargs: ', kwargs)

    # raise NotImplementedError

    instance_store = InstanceStoreSingleton()
    del instance_store._shared_instance_state["assistants"][assistant_id]

    return AssistantDeleted(
        id=assistant_id,
        deleted=True,
        object="assistant.deleted",
    ).model_dump(mode="json")


# def delete_message(thread_id, message_id):  # noqa: E501
async def delete_message(*args, **kwargs):
    """Deletes a message.

     # noqa: E501

    :param thread_id: The ID of the thread to which this message belongs.
    :type thread_id: str
    :param message_id: The ID of the message to delete.
    :type message_id: str

    :rtype: Union[DeleteMessageResponse, Tuple[DeleteMessageResponse, int], Tuple[DeleteMessageResponse, int, Dict[str, str]]
    """
    print("=== delete_message ===")
    print("args: ", args)
    print("kwargs: ", kwargs)

    raise NotImplementedError


# def delete_thread(thread_id):  # noqa: E501
async def delete_thread(*args, **kwargs):
    """Delete a thread.

     # noqa: E501

    :param thread_id: The ID of the thread to delete.
    :type thread_id: str

    :rtype: Union[DeleteThreadResponse, Tuple[DeleteThreadResponse, int], Tuple[DeleteThreadResponse, int, Dict[str, str]]
    """
    print("=== delete_thread ===")
    print("args: ", args)
    print("kwargs: ", kwargs)

    raise NotImplementedError


# def get_assistant(assistant_id):  # noqa: E501
async def get_assistant(assistant_id, token_info, user):
    """Retrieves an assistant.

     # noqa: E501

    :param assistant_id: The ID of the assistant to retrieve.
    :type assistant_id: str

    :rtype: Union[AssistantObject, Tuple[AssistantObject, int], Tuple[AssistantObject, int, Dict[str, str]]
    """
    print("=== get_assistant ===")

    return (
        InstanceStoreSingleton()
        ._shared_instance_state["assistants"]
        .get(assistant_id)
        .model_dump(mode="json")
    )


# def get_message(thread_id, message_id):  # noqa: E501
async def get_message(*args, **kwargs):
    """Retrieve a message.

     # noqa: E501

    :param thread_id: The ID of the [thread](/docs/api-reference/threads) to which this message belongs.
    :type thread_id: str
    :param message_id: The ID of the message to retrieve.
    :type message_id: str

    :rtype: Union[MessageObject, Tuple[MessageObject, int], Tuple[MessageObject, int, Dict[str, str]]
    """
    print("=== get_message ===")
    print("args: ", args)
    print("kwargs: ", kwargs)

    raise NotImplementedError


# def get_run(thread_id, run_id):  # noqa: E501
async def get_run(run_id, thread_id, token_info, user):
    """Retrieves a run.

     # noqa: E501

    :param thread_id: The ID of the [thread](/docs/api-reference/threads) that was run.
    :type thread_id: str
    :param run_id: The ID of the run to retrieve.
    :type run_id: str

    :rtype: Union[RunObject, Tuple[RunObject, int], Tuple[RunObject, int, Dict[str, str]]
    """
    print("=== get_run ===")

    return (
        InstanceStoreSingleton()
        ._shared_instance_state["runs"]
        .get(run_id)
        .model_dump(mode="json")
    )


# def get_run_step(thread_id, run_id, step_id):  # noqa: E501
async def get_run_step(*args, **kwargs):
    """Retrieves a run step.

     # noqa: E501

    :param thread_id: The ID of the thread to which the run and run step belongs.
    :type thread_id: str
    :param run_id: The ID of the run to which the run step belongs.
    :type run_id: str
    :param step_id: The ID of the run step to retrieve.
    :type step_id: str

    :rtype: Union[RunStepObject, Tuple[RunStepObject, int], Tuple[RunStepObject, int, Dict[str, str]]
    """
    print("=== get_run_step ===")
    print("args: ", args)
    print("kwargs: ", kwargs)

    raise NotImplementedError


# def get_thread(thread_id):  # noqa: E501
async def get_thread(token_info, thread_id, user):
    """Retrieves a thread.

     # noqa: E501

    :param thread_id: The ID of the thread to retrieve.
    :type thread_id: str

    :rtype: Union[ThreadObject, Tuple[ThreadObject, int], Tuple[ThreadObject, int, Dict[str, str]]
    """
    print("=== get_thread ===")
    instance_store = InstanceStoreSingleton()
    thread = instance_store._shared_instance_state["threads"][thread_id]

    return thread.model_dump(mode="json")


# def list_assistants(limit=None, order=None, after=None, before=None):  # noqa: E501
async def list_assistants(*args, **kwargs):
    """Returns a list of assistants.

     # noqa: E501

    :param limit: A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 20.
    :type limit: int
    :param order: Sort order by the &#x60;created_at&#x60; timestamp of the objects. &#x60;asc&#x60; for ascending order and &#x60;desc&#x60; for descending order.
    :type order: str
    :param after: A cursor for use in pagination. &#x60;after&#x60; is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with obj_foo, your subsequent call can include after&#x3D;obj_foo in order to fetch the next page of the list.
    :type after: str
    :param before: A cursor for use in pagination. &#x60;before&#x60; is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with obj_foo, your subsequent call can include before&#x3D;obj_foo in order to fetch the previous page of the list.
    :type before: str

    :rtype: Union[ListAssistantsResponse, Tuple[ListAssistantsResponse, int], Tuple[ListAssistantsResponse, int, Dict[str, str]]
    """
    print("=== list_assistants ===")
    print("args: ", args)
    print("kwargs: ", kwargs)

    raise NotImplementedError


# def list_messages(thread_id, limit=None, order=None, after=None, before=None, run_id=None):  # noqa: E501
async def list_messages(
    token_info: dict,
    thread_id: str,
    user: str,
    after: Optional[str] = None,
    before: Optional[str] = None,
    limit: int = 20,
    order: str = "desc",
):
    """Returns a list of messages for a given thread.

     # noqa: E501

    :param thread_id: The ID of the [thread](/docs/api-reference/threads) the messages belong to.
    :type thread_id: str
    :param limit: A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 20.
    :type limit: int
    :param order: Sort order by the &#x60;created_at&#x60; timestamp of the objects. &#x60;asc&#x60; for ascending order and &#x60;desc&#x60; for descending order.
    :type order: str
    :param after: A cursor for use in pagination. &#x60;after&#x60; is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with obj_foo, your subsequent call can include after&#x3D;obj_foo in order to fetch the next page of the list.
    :type after: str
    :param before: A cursor for use in pagination. &#x60;before&#x60; is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with obj_foo, your subsequent call can include before&#x3D;obj_foo in order to fetch the previous page of the list.
    :type before: str
    :param run_id: Filter messages by the run ID that generated them.
    :type run_id: str

    :rtype: Union[ListMessagesResponse, Tuple[ListMessagesResponse, int], Tuple[ListMessagesResponse, int, Dict[str, str]]
    """
    print("=== list_messages ===")
    print("after: ", after)
    print("before: ", before)
    print("limit: ", limit)
    print("order: ", order)
    print("token_info: ", token_info)
    print("thread_id: ", thread_id)
    print("user: ", user)

    # thread: Thread = Thread(**await get_thread(token_info=token_info, thread_id=thread_id, user=user))

    after_created_at = (
        InstanceStoreSingleton()._shared_instance_state["messages"][after].created_at
        if after
        else 0
    )
    before_created_at = (
        InstanceStoreSingleton()._shared_instance_state["messages"][before].created_at
        if before
        else int(time.time())
    )
    limit = None if limit == -1 else limit

    # def message_filter(message: Message):
    #     return message.created_at > after_created_at and message.created_at < before_created_at and message.thread_id == thread_id

    # filtered_messages = list(filter(
    #     function=message_filter,
    #     iterable=InstanceStoreSingleton()._shared_instance_state["messages"].values(),
    # ))

    # filtered_messages =

    messages: List[Message] = sorted(
        [
            message
            for message in InstanceStoreSingleton()
            ._shared_instance_state["messages"]
            .values()
            if message.created_at > after_created_at
            and message.created_at < before_created_at
            and message.thread_id == thread_id
        ],
        key=lambda message: message.created_at,
        reverse=True if order == "desc" else False,
    )[0:limit]

    # TODO: handle AsyncCursoPage as well

    return AsyncCursorPage(
        # return SyncCursorPage(
        data=messages,
    ).model_dump(mode="json")


# def list_run_steps(thread_id, run_id, limit=None, order=None, after=None, before=None):  # noqa: E501
async def list_run_steps(*args, **kwargs):
    """Returns a list of run steps belonging to a run.

     # noqa: E501

    :param thread_id: The ID of the thread the run and run steps belong to.
    :type thread_id: str
    :param run_id: The ID of the run the run steps belong to.
    :type run_id: str
    :param limit: A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 20.
    :type limit: int
    :param order: Sort order by the &#x60;created_at&#x60; timestamp of the objects. &#x60;asc&#x60; for ascending order and &#x60;desc&#x60; for descending order.
    :type order: str
    :param after: A cursor for use in pagination. &#x60;after&#x60; is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with obj_foo, your subsequent call can include after&#x3D;obj_foo in order to fetch the next page of the list.
    :type after: str
    :param before: A cursor for use in pagination. &#x60;before&#x60; is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with obj_foo, your subsequent call can include before&#x3D;obj_foo in order to fetch the previous page of the list.
    :type before: str

    :rtype: Union[ListRunStepsResponse, Tuple[ListRunStepsResponse, int], Tuple[ListRunStepsResponse, int, Dict[str, str]]
    """
    print("=== list_run_steps ===")
    print("args: ", args)
    print("kwargs: ", kwargs)

    raise NotImplementedError


# def list_runs(thread_id, limit=None, order=None, after=None, before=None):  # noqa: E501
async def list_runs(*args, **kwargs):
    """Returns a list of runs belonging to a thread.

     # noqa: E501

    :param thread_id: The ID of the thread the run belongs to.
    :type thread_id: str
    :param limit: A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 20.
    :type limit: int
    :param order: Sort order by the &#x60;created_at&#x60; timestamp of the objects. &#x60;asc&#x60; for ascending order and &#x60;desc&#x60; for descending order.
    :type order: str
    :param after: A cursor for use in pagination. &#x60;after&#x60; is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with obj_foo, your subsequent call can include after&#x3D;obj_foo in order to fetch the next page of the list.
    :type after: str
    :param before: A cursor for use in pagination. &#x60;before&#x60; is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with obj_foo, your subsequent call can include before&#x3D;obj_foo in order to fetch the previous page of the list.
    :type before: str

    :rtype: Union[ListRunsResponse, Tuple[ListRunsResponse, int], Tuple[ListRunsResponse, int, Dict[str, str]]
    """
    print("=== list_runs ===")
    print("args: ", args)
    print("kwargs: ", kwargs)

    raise NotImplementedError


# def modify_assistant(assistant_id, modify_assistant_request):  # noqa: E501
async def modify_assistant(
    assistant_id: str, body: AssistantUpdateParams, token_info, user
):
    """Modifies an assistant.

     # noqa: E501

    :param assistant_id: The ID of the assistant to modify.
    :type assistant_id: str
    :param modify_assistant_request:
    :type modify_assistant_request: dict | bytes

    :rtype: Union[AssistantObject, Tuple[AssistantObject, int], Tuple[AssistantObject, int, Dict[str, str]]
    """
    # if connexion.request.is_json:
    #     modify_assistant_request = ModifyAssistantRequest.from_dict(connexion.request.get_json())  # noqa: E501

    print("=== modify_assistant ===")
    print("assistant_id: ", assistant_id)
    print("body: ", body)

    assistant: Assistant = Assistant(
        **await get_assistant(
            assistant_id=assistant_id, token_info=token_info, user=user
        )
    )

    # assistant = Assistant(
    #     id=str(uuid.uuid4()),
    #     created_at=int(time.time()),
    #     description=body.get("description"),
    #     instructions=body.get("instructions"),
    #     model=body.get("model"),
    #     name=body.get("name"),
    #     object="assistant",
    #     tools=body.get("tools", [])
    # )

    print("assistant [before]: ", assistant)

    assistant.description = body.get("description", assistant.description)
    assistant.instructions = body.get("instructions", assistant.instructions)
    assistant.model = body.get("model", assistant.model)
    assistant.name = body.get("name", assistant.name)
    assistant.tools = body.get("tools", assistant.tools)

    print("assistant [after]: ", assistant)

    instance_store = InstanceStoreSingleton()
    instance_store._shared_instance_state["assistants"][assistant.id] = assistant

    return await get_assistant(
        assistant_id=assistant_id, token_info=token_info, user=user
    )


# def modify_message(thread_id, message_id, modify_message_request):  # noqa: E501
async def modify_message(*args, **kwargs):
    """Modifies a message.

     # noqa: E501

    :param thread_id: The ID of the thread to which this message belongs.
    :type thread_id: str
    :param message_id: The ID of the message to modify.
    :type message_id: str
    :param modify_message_request:
    :type modify_message_request: dict | bytes

    :rtype: Union[MessageObject, Tuple[MessageObject, int], Tuple[MessageObject, int, Dict[str, str]]
    """
    # if connexion.request.is_json:
    #     modify_message_request = ModifyMessageRequest.from_dict(connexion.request.get_json())  # noqa: E501

    print("=== modify_message ===")
    print("args: ", args)
    print("kwargs: ", kwargs)

    raise NotImplementedError


# def modify_run(thread_id, run_id, modify_run_request):  # noqa: E501
async def modify_run(modify_run_request, run_id, token_info, thread_id, user, **kwargs):
    """Modifies a run.

     # noqa: E501

    :param thread_id: The ID of the [thread](/docs/api-reference/threads) that was run.
    :type thread_id: str
    :param run_id: The ID of the run to modify.
    :type run_id: str
    :param modify_run_request:
    :type modify_run_request: dict | bytes

    :rtype: Union[RunObject, Tuple[RunObject, int], Tuple[RunObject, int, Dict[str, str]]
    """
    # if connexion.request.is_json:
    #     modify_run_request = ModifyRunRequest.from_dict(connexion.request.get_json())  # noqa: E501

    print("=== modify_run ===")
    print("modify_run_request: ", modify_run_request)
    # print('args: ', args)
    print("kwargs: ", kwargs)

    run: Run = Run(
        **await get_run(
            run_id=run_id, token_info=token_info, thread_id=thread_id, user=user
        )
    )

    run.status = modify_run_request.get("status")

    InstanceStoreSingleton()._shared_instance_state["runs"][run_id] = run

    return run.model_dump(mode="json")


# def modify_thread(thread_id, modify_thread_request):  # noqa: E501
async def modify_thread(*args, **kwargs):
    """Modifies a thread.

     # noqa: E501

    :param thread_id: The ID of the thread to modify. Only the &#x60;metadata&#x60; can be modified.
    :type thread_id: str
    :param modify_thread_request:
    :type modify_thread_request: dict | bytes

    :rtype: Union[ThreadObject, Tuple[ThreadObject, int], Tuple[ThreadObject, int, Dict[str, str]]
    """
    # if connexion.request.is_json:
    #     modify_thread_request = ModifyThreadRequest.from_dict(connexion.request.get_json())  # noqa: E501

    print("=== modify_thread ===")
    print("args: ", args)
    print("kwargs: ", kwargs)

    raise NotImplementedError


async def submit_tool_ouputs_to_run(
    *args, **kwargs
):  # NOTE: the typo in outputs is b/c the auto-generated openai-openapi spec has that typo
    """When a run has the &#x60;status: \&quot;requires_action\&quot;&#x60; and &#x60;required_action.type&#x60; is &#x60;submit_tool_outputs&#x60;, this endpoint can be used to submit the outputs from the tool calls once they&#39;re all completed. All outputs must be submitted in a single request.

    :param thread_id: The ID of the [thread](/docs/api-reference/threads) to which this run belongs.
    :type thread_id: str
    :param run_id: The ID of the run that requires the tool output submission.
    :type run_id: str
    :param submit_tool_outputs_run_request:
    :type submit_tool_outputs_run_request: dict | bytes

    :rtype: Union[RunObject, Tuple[RunObject, int], Tuple[RunObject, int, Dict[str, str]]
    """
    # if connexion.request.is_json:
    #     submit_tool_outputs_run_request = SubmitToolOutputsRunRequest.from_dict(connexion.request.get_json())  # noqa: E501

    print("=== submit_tool_outputs_to_run ===")
    print("args: ", args)
    print("kwargs: ", kwargs)

    raise NotImplementedError



import asyncio

from openai import AsyncOpenAI, AsyncStream
from openai.types.chat.chat_completion import ChatCompletion
from openai.types.chat.chat_completion_chunk import ChatCompletionChunk
from openai.types.chat.completion_create_params import CompletionCreateParams
from sse_starlette import EventSourceResponse


async def create_chat_completion(
    body: CompletionCreateParams, token_info: dict, user: str, **kwargs
):
    """Creates a model response for the given chat conversation.

     # noqa: E501

    :param create_chat_completion_request:
    :type create_chat_completion_request: dict | bytes

    :rtype: Union[CreateChatCompletionResponse, Tuple[CreateChatCompletionResponse, int], Tuple[CreateChatCompletionResponse, int, Dict[str, str]]
    """
    client = AsyncOpenAI(
        api_key="sk-no-key-required",
        base_url="http://localhost:8080/v1",
    )

    chat_completion: ChatCompletion | AsyncStream[ChatCompletionChunk] = (
        await client.chat.completions.create(
            **body,
            user=user,
        )
    )

    if type(chat_completion) == AsyncStream:

        async def event_publisher():
            try:
                async for chunk in chat_completion:
                    yield chunk.model_dump_json()

            except asyncio.CancelledError as e:
                print(f"Disconnected from client (via refresh/close)")
                # Do any other cleanup, if any
                raise e

        return EventSourceResponse(event_publisher())

    else:
        return chat_completion.model_dump(mode="json")



import time
import uuid
from typing import List, Optional

from openai.types.beta.thread import Thread
from openai.types.beta.threads.message import Message, MessageContent
from openai.types.beta.threads.text import Text
from openai.types.beta.threads.text_content_block import TextContentBlock


# def create_message(thread_id, create_message_request):  # noqa: E501
async def create_thread_message(body, token_info, thread_id, user):
    """Create a message.

     # noqa: E501

    :param thread_id: The ID of the [thread](/docs/api-reference/threads) to create a message for.
    :type thread_id: str
    :param create_message_request:
    :type create_message_request: dict | bytes

    :rtype: Union[MessageObject, Tuple[MessageObject, int], Tuple[MessageObject, int, Dict[str, str]]
    """
    # if connexion.request.is_json:
    #     create_message_request = CreateMessageRequest.from_dict(connexion.request.get_json())  # noqa: E501

    print("=== create_message ===")

    # thread = instance_store._shared_instance_state["threads"][thread_id]
    thread: Thread = Thread(
        **await get_thread(token_info=token_info, thread_id=thread_id, user=user)
    )

    content: List[MessageContent] = []

    if body.get("content"):
        content.append(
            TextContentBlock(
                text=Text(
                    annotations=[],
                    value=(
                        body.get("content")
                        if type(body.get("content")) == str
                        else body.get("content")[0].get("text")
                    ),
                ),
                type="text",
            )
        )

    message = Message(
        id=str(uuid.uuid4()),
        attachments=[],
        content=content,
        created_at=int(time.time()),
        object="thread.message",
        role=body.get("role"),
        thread_id=thread.id,
        status="incomplete",
    )

    print("message: ", message)

    instance_store = InstanceStoreSingleton()
    instance_store._shared_instance_state["messages"][message.id] = message

    # return message.model_dump(mode="json")
    return message


# def get_thread(thread_id):  # noqa: E501
async def get_thread(token_info, thread_id, user):
    """Retrieves a thread.

     # noqa: E501

    :param thread_id: The ID of the thread to retrieve.
    :type thread_id: str

    :rtype: Union[ThreadObject, Tuple[ThreadObject, int], Tuple[ThreadObject, int, Dict[str, str]]
    """
    print("=== get_thread ===")
    instance_store = InstanceStoreSingleton()
    thread = instance_store._shared_instance_state["threads"][thread_id]

    # return thread.model_dump(mode="json")
    return thread


async def get_thread_messages(
    token_info: dict,
    thread_id: str,
    user: str,
    after: Optional[str] = None,
    before: Optional[str] = None,
    limit: int = 20,
    order: str = "desc",
):
    """Returns a list of messages for a given thread.

     # noqa: E501

    :param thread_id: The ID of the [thread](/docs/api-reference/threads) the messages belong to.
    :type thread_id: str
    :param limit: A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 20.
    :type limit: int
    :param order: Sort order by the &#x60;created_at&#x60; timestamp of the objects. &#x60;asc&#x60; for ascending order and &#x60;desc&#x60; for descending order.
    :type order: str
    :param after: A cursor for use in pagination. &#x60;after&#x60; is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with obj_foo, your subsequent call can include after&#x3D;obj_foo in order to fetch the next page of the list.
    :type after: str
    :param before: A cursor for use in pagination. &#x60;before&#x60; is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with obj_foo, your subsequent call can include before&#x3D;obj_foo in order to fetch the previous page of the list.
    :type before: str
    :param run_id: Filter messages by the run ID that generated them.
    :type run_id: str

    :rtype: Union[ListMessagesResponse, Tuple[ListMessagesResponse, int], Tuple[ListMessagesResponse, int, Dict[str, str]]
    """
    print("=== list_messages ===")
    print("after: ", after)
    print("before: ", before)
    print("limit: ", limit)
    print("order: ", order)
    print("token_info: ", token_info)
    print("thread_id: ", thread_id)
    print("user: ", user)

    # thread: Thread = Thread(**await get_thread(token_info=token_info, thread_id=thread_id, user=user))

    after_created_at = (
        InstanceStoreSingleton()._shared_instance_state["messages"][after].created_at
        if after
        else 0
    )
    before_created_at = (
        InstanceStoreSingleton()._shared_instance_state["messages"][before].created_at
        if before
        else int(time.time())
    )
    limit = None if limit == -1 else limit

    # def message_filter(message: Message):
    #     return message.created_at > after_created_at and message.created_at < before_created_at and message.thread_id == thread_id

    # filtered_messages = list(filter(
    #     function=message_filter,
    #     iterable=InstanceStoreSingleton()._shared_instance_state["messages"].values(),
    # ))

    # filtered_messages =

    messages: List[Message] = sorted(
        [
            message
            for message in InstanceStoreSingleton()
            ._shared_instance_state["messages"]
            .values()
            if message.created_at > after_created_at
            and message.created_at < before_created_at
            and message.thread_id == thread_id
        ],
        key=lambda message: message.created_at,
        reverse=True if order == "desc" else False,
    )[0:limit]

    # TODO: handle AsyncCursoPage as well

    # return AsyncCursorPage(
    # # return SyncCursorPage(
    #     data=messages,
    # ).model_dump(mode="json")

    return messages


async def mutate_thread(*args, **kwargs):
    raise NotImplementedError


async def query_thread(*args, **kwargs):
    raise NotImplementedError



import enum
import uuid
from typing import List, Optional

from openai.types.fine_tuning.fine_tuning_job import FineTuningJob
from pydantic import BaseModel

# from sqlalchemy import JSON
# import sqlalchemy
from sqlalchemy import Column, Enum

# from sqlalchemy.dialects.sqlite.json import JSON
from sqlalchemy.types import JSON
from sqlmodel import Field, Session, SQLModel

# from timestep.database import InstanceStoreSingleton


# class FineTuningJobErrorSQLModel(Error, Column(JSON)):
#   pass
#   __tablename__: str = "fine_tuning_job_errors"

#   id: uuid.UUID = Field(default_factory=uuid.uuid4, primary_key=True)


class FineTuningJobStatus(str, enum.Enum):
    cancelled = "cancelled"
    failed = "failed"
    queued = "queued"
    running = "running"
    succeeded = "succeeded"
    validating_files = "validating_files"


# TODO: can i just use the flow run plus artifacts for this?
class FineTuningJobSQLModel(FineTuningJob, SQLModel, table=True):
    __tablename__: str = "fine_tuning_jobs"
    object: str = "fine_tuning.job"
    id: uuid.UUID = Field(default_factory=uuid.uuid4, primary_key=True)
    # id: uuid.uuid4 = Field(default_factory=uuid.uuid4, primary_key=True)

    error: Optional[dict] = Field(sa_column=Column(JSON), default=JSON.NULL)
    integrations: Optional[List[dict]] = Field(
        sa_column=Column(JSON), default=JSON.NULL
    )
    hyperparameters: dict = Field(sa_column=Column(JSON), default={})
    result_files: List[str] = Field(sa_column=Column(JSON), default=[])
    status: FineTuningJobStatus = Field(
        sa_column=Column(Enum(FineTuningJobStatus)), default=FineTuningJobStatus.queued
    )


class ModelInstanceStoreSingleton(object):
    models: dict[uuid.UUID] = {}
    _shared_instance_state = {
        "models": {},
    }

    def __new__(cls, *args, **kwargs):
        obj = super(ModelInstanceStoreSingleton, cls).__new__(cls, *args, **kwargs)
        obj.__dict__ = cls._shared_instance_state

        return obj

    def insert(self, base_model: BaseModel):
        if type(base_model) is FineTuningJob:
            print("=== insert ===")
            print("id: ", base_model.id)
            print('base_model.model_dump()["id"]', base_model.model_dump()["id"])
            fine_tuning_job = FineTuningJobSQLModel.model_validate(
                base_model.model_dump()
            )
            print("fine_tuning_job.id: ", fine_tuning_job.id)

            with Session(engine) as session:
                session.add(fine_tuning_job)
                session.commit()

        else:
            raise NotImplementedError(f"Type {type(base_model)} is not yet implemented")

    def select(self, base_model_class, id: str):
        print("=== select ===")
        print("id: ", id)

        if issubclass(base_model_class, FineTuningJob):
            with Session(engine) as session:
                fine_tuning_job = session.get(
                    FineTuningJobSQLModel, ident=uuid.UUID(id, version=4)
                )
                print("fine_tuning_job.id: ", fine_tuning_job.id)

                assert (
                    type(fine_tuning_job.id) == uuid.UUID
                ), f"{type(fine_tuning_job.id)} != uuid.UUID"

                # m = base_model_class.model_validate(fine_tuning_job, strict=True)
                # print('type(m): ', type(m))
                #

                return FineTuningJob(**fine_tuning_job.model_dump(mode="json"))

                # raise NotImplementedError

        else:
            raise NotImplementedError(f"Type {base_model_class} is not yet implemented")


model_instance_store = InstanceStoreSingleton()
model_instance_store.shared_variable = "Shared Variable"



async def cancel_vector_store_file_batch(vector_store_id, batch_id):  # noqa: E501
    """Cancel a vector store file batch. This attempts to cancel the processing of files in this batch as soon as possible.

     # noqa: E501

    :param vector_store_id: The ID of the vector store that the file batch belongs to.
    :type vector_store_id: str
    :param batch_id: The ID of the file batch to cancel.
    :type batch_id: str

    :rtype: Union[VectorStoreFileBatchObject, Tuple[VectorStoreFileBatchObject, int], Tuple[VectorStoreFileBatchObject, int, Dict[str, str]]
    """
    raise NotImplementedError


async def create_vector_store(create_vector_store_request):  # noqa: E501
    """Create a vector store.

     # noqa: E501

    :param create_vector_store_request:
    :type create_vector_store_request: dict | bytes

    :rtype: Union[VectorStoreObject, Tuple[VectorStoreObject, int], Tuple[VectorStoreObject, int, Dict[str, str]]
    """
    raise NotImplementedError


async def create_vector_store_file(
    vector_store_id, create_vector_store_file_request
):  # noqa: E501
    """Create a vector store file by attaching a [File](/docs/api-reference/files) to a [vector store](/docs/api-reference/vector-stores/object).

     # noqa: E501

    :param vector_store_id: The ID of the vector store for which to create a File.
    :type vector_store_id: str
    :param create_vector_store_file_request:
    :type create_vector_store_file_request: dict | bytes

    :rtype: Union[VectorStoreFileObject, Tuple[VectorStoreFileObject, int], Tuple[VectorStoreFileObject, int, Dict[str, str]]
    """
    raise NotImplementedError


async def create_vector_store_file_batch(
    vector_store_id, create_vector_store_file_batch_request
):  # noqa: E501
    """Create a vector store file batch.

     # noqa: E501

    :param vector_store_id: The ID of the vector store for which to create a File Batch.
    :type vector_store_id: str
    :param create_vector_store_file_batch_request:
    :type create_vector_store_file_batch_request: dict | bytes

    :rtype: Union[VectorStoreFileBatchObject, Tuple[VectorStoreFileBatchObject, int], Tuple[VectorStoreFileBatchObject, int, Dict[str, str]]
    """
    raise NotImplementedError


async def delete_vector_store(vector_store_id):  # noqa: E501
    """Delete a vector store.

     # noqa: E501

    :param vector_store_id: The ID of the vector store to delete.
    :type vector_store_id: str

    :rtype: Union[DeleteVectorStoreResponse, Tuple[DeleteVectorStoreResponse, int], Tuple[DeleteVectorStoreResponse, int, Dict[str, str]]
    """
    raise NotImplementedError


async def delete_vector_store_file(vector_store_id, file_id):  # noqa: E501
    """Delete a vector store file. This will remove the file from the vector store but the file itself will not be deleted. To delete the file, use the [delete file](/docs/api-reference/files/delete) endpoint.

     # noqa: E501

    :param vector_store_id: The ID of the vector store that the file belongs to.
    :type vector_store_id: str
    :param file_id: The ID of the file to delete.
    :type file_id: str

    :rtype: Union[DeleteVectorStoreFileResponse, Tuple[DeleteVectorStoreFileResponse, int], Tuple[DeleteVectorStoreFileResponse, int, Dict[str, str]]
    """
    raise NotImplementedError


async def get_vector_store(vector_store_id):  # noqa: E501
    """Retrieves a vector store.

     # noqa: E501

    :param vector_store_id: The ID of the vector store to retrieve.
    :type vector_store_id: str

    :rtype: Union[VectorStoreObject, Tuple[VectorStoreObject, int], Tuple[VectorStoreObject, int, Dict[str, str]]
    """
    raise NotImplementedError


async def get_vector_store_file(vector_store_id, file_id):  # noqa: E501
    """Retrieves a vector store file.

     # noqa: E501

    :param vector_store_id: The ID of the vector store that the file belongs to.
    :type vector_store_id: str
    :param file_id: The ID of the file being retrieved.
    :type file_id: str

    :rtype: Union[VectorStoreFileObject, Tuple[VectorStoreFileObject, int], Tuple[VectorStoreFileObject, int, Dict[str, str]]
    """
    raise NotImplementedError


async def get_vector_store_file_batch(vector_store_id, batch_id):  # noqa: E501
    """Retrieves a vector store file batch.

     # noqa: E501

    :param vector_store_id: The ID of the vector store that the file batch belongs to.
    :type vector_store_id: str
    :param batch_id: The ID of the file batch being retrieved.
    :type batch_id: str

    :rtype: Union[VectorStoreFileBatchObject, Tuple[VectorStoreFileBatchObject, int], Tuple[VectorStoreFileBatchObject, int, Dict[str, str]]
    """
    raise NotImplementedError


async def list_files_in_vector_store_batch(
    vector_store_id,
    batch_id,
    limit=None,
    order=None,
    after=None,
    before=None,
    filter=None,
):  # noqa: E501
    """Returns a list of vector store files in a batch.

     # noqa: E501

    :param vector_store_id: The ID of the vector store that the files belong to.
    :type vector_store_id: str
    :param batch_id: The ID of the file batch that the files belong to.
    :type batch_id: str
    :param limit: A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 20.
    :type limit: int
    :param order: Sort order by the &#x60;created_at&#x60; timestamp of the objects. &#x60;asc&#x60; for ascending order and &#x60;desc&#x60; for descending order.
    :type order: str
    :param after: A cursor for use in pagination. &#x60;after&#x60; is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with obj_foo, your subsequent call can include after&#x3D;obj_foo in order to fetch the next page of the list.
    :type after: str
    :param before: A cursor for use in pagination. &#x60;before&#x60; is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with obj_foo, your subsequent call can include before&#x3D;obj_foo in order to fetch the previous page of the list.
    :type before: str
    :param filter: Filter by file status. One of &#x60;in_progress&#x60;, &#x60;completed&#x60;, &#x60;failed&#x60;, &#x60;cancelled&#x60;.
    :type filter: str

    :rtype: Union[ListVectorStoreFilesResponse, Tuple[ListVectorStoreFilesResponse, int], Tuple[ListVectorStoreFilesResponse, int, Dict[str, str]]
    """
    raise NotImplementedError


async def list_vector_store_files(
    vector_store_id, limit=None, order=None, after=None, before=None, filter=None
):  # noqa: E501
    """Returns a list of vector store files.

     # noqa: E501

    :param vector_store_id: The ID of the vector store that the files belong to.
    :type vector_store_id: str
    :param limit: A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 20.
    :type limit: int
    :param order: Sort order by the &#x60;created_at&#x60; timestamp of the objects. &#x60;asc&#x60; for ascending order and &#x60;desc&#x60; for descending order.
    :type order: str
    :param after: A cursor for use in pagination. &#x60;after&#x60; is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with obj_foo, your subsequent call can include after&#x3D;obj_foo in order to fetch the next page of the list.
    :type after: str
    :param before: A cursor for use in pagination. &#x60;before&#x60; is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with obj_foo, your subsequent call can include before&#x3D;obj_foo in order to fetch the previous page of the list.
    :type before: str
    :param filter: Filter by file status. One of &#x60;in_progress&#x60;, &#x60;completed&#x60;, &#x60;failed&#x60;, &#x60;cancelled&#x60;.
    :type filter: str

    :rtype: Union[ListVectorStoreFilesResponse, Tuple[ListVectorStoreFilesResponse, int], Tuple[ListVectorStoreFilesResponse, int, Dict[str, str]]
    """
    raise NotImplementedError


async def list_vector_stores(
    limit=None, order=None, after=None, before=None
):  # noqa: E501
    """Returns a list of vector stores.

     # noqa: E501

    :param limit: A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 20.
    :type limit: int
    :param order: Sort order by the &#x60;created_at&#x60; timestamp of the objects. &#x60;asc&#x60; for ascending order and &#x60;desc&#x60; for descending order.
    :type order: str
    :param after: A cursor for use in pagination. &#x60;after&#x60; is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with obj_foo, your subsequent call can include after&#x3D;obj_foo in order to fetch the next page of the list.
    :type after: str
    :param before: A cursor for use in pagination. &#x60;before&#x60; is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with obj_foo, your subsequent call can include before&#x3D;obj_foo in order to fetch the previous page of the list.
    :type before: str

    :rtype: Union[ListVectorStoresResponse, Tuple[ListVectorStoresResponse, int], Tuple[ListVectorStoresResponse, int, Dict[str, str]]
    """
    raise NotImplementedError


async def modify_vector_store(
    vector_store_id, update_vector_store_request
):  # noqa: E501
    """Modifies a vector store.

     # noqa: E501

    :param vector_store_id: The ID of the vector store to modify.
    :type vector_store_id: str
    :param update_vector_store_request:
    :type update_vector_store_request: dict | bytes

    :rtype: Union[VectorStoreObject, Tuple[VectorStoreObject, int], Tuple[VectorStoreObject, int, Dict[str, str]]
    """
    raise NotImplementedError
